#!/usr/bin/env node

const chalk = require('chalk');
const program = require('commander');
const { existsSync: exists } = require('fs');
const path = require('path');
const home = require('os').homedir();
const inquirer = require('inquirer');
const sao = require('sao');

let rawTemplate;
let rawPath;

/**
 * Usage
 */
program
  .arguments('<template>')
  .arguments('[path-to-project]')
  .action(function(template, path) {
    rawTemplate = template;
    if (path) {
      rawPath = path;
    } else {
      rawPath = '.';
    }
  })
  .option(
    '-c, --npm-client <client>',
    "specify npm client for installing packages ('tnpm' | 'yarn')"
  )
  .option(
    '-r, --registry <registry>',
    'use a custom registry for package manager'
  ).usage(`
  $ marmot create <template> [path-to-project] [options]`);

/**
 * Help
 */
program.on('--help', () => {
  console.log(`
Creates new projects from create-* template.

Examples:
  ${chalk.gray('# create project with create-h5-app template')}
  $ marmot create h5-app path/to/project

  ${chalk.gray(
    '# create project with create-h5-app template and specify npm client'
  )}
  $ marmot create h5-app path/to/project -c tnpm

  ${chalk.gray(
    '# create project in current directory with create-h5-app template'
  )}
  $ marmot create h5-app
`);
});

/**
 * Parse
 */
program.parse(process.argv);

/**
 * Validation
 */
if (typeof rawTemplate === 'undefined') {
  program.outputHelp();
  process.exit(1);
}

// resolve home dir
rawPath = rawPath.replace(/^~/, home);
const outDir = path.resolve(rawPath);

/**
 * Padding
 */
process.on('exit', () => {
  console.log();
});

/**
 * Specify npm client and registry
 */
let npmClient = {};
let registry = {};
if (program.npmClient) {
  npmClient = { npmClient: program.npmClient };
}
if (program.registry) {
  registry = { registry: program.registry };
}

/**
 * Questions
 */
const questions = [];

if (exists(outDir)) {
  questions.push({
    type: 'confirm',
    name: 'override',
    message: `The directory ${outDir} already exists, continue?`,
    default: false,
  });
}

/**
 * Prompt and Generate
 */
async function run() {
  const res = await inquirer.prompt(questions);
  if (res.override === false) {
    process.exit(1);
  }
  let fullTemplate;
  const templateParts = rawTemplate.split('/');
  if (templateParts.length === 1) {
    fullTemplate = `create-${rawTemplate}`;
  } else if (templateParts.length === 2) {
    fullTemplate = `${templateParts[0]}/create-${templateParts[1]}`;
  } else {
    console.log(chalk.red('Template format invalid'));
  }
  console.log(
    `Generating project at ${outDir}, using template ${chalk.blue(
      fullTemplate
    )}`
  );
  // https://saojs.org/guide/getting-started.html#using-generators
  const generator = `npm:${fullTemplate}`;
  const options = {
    generator,
    outDir,
    ...npmClient,
    ...registry,
    update: true,
  };
  const app = sao(options);
  await app.run();
}

run();
