#!/usr/bin/env node

const home = require('os').homedir();
const path = require('path');
const ora = require('ora');
const mkdirp = require('mkdirp');
const { existsSync, writeFileSync } = require('fs');
const { pki } = require('node-forge');
const program = require('commander');
const inquirer = require('inquirer');
const { success, error } = require('../lib/logger');

const { rsa } = pki;

const DEFAULT_PATH = path.join(home, '.marmot/rsa-key/');

/**
 * Help
 */
program
  .name(
    `
$ marmot genkey`,
  )
  .usage('')
  .on('--help', function() {
    console.log(`
Generate a new RSA private key that can be used for signing at "~/.marmot/rsa-key/".
`);
  });

/**
 * Parse
 */
program.parse(process.argv);

preCheck();

function preCheck() {
  if (existsSync(DEFAULT_PATH)) {
    const message = `${DEFAULT_PATH} is not empty, overwrite the path ?`;
    inquirer
      .prompt([
        {
          type: 'confirm',
          message: message,
          name: 'confirm',
        },
      ])
      .then(answers => {
        answers.confirm && entry();
      });
  } else {
    entry();
  }
}

function entry() {
  mkdirp.sync(DEFAULT_PATH);
  const targetPath = DEFAULT_PATH;
  const spinner = ora(`generate rsa key pairs, path: ${targetPath}`);

  spinner.start();

  rsa.generateKeyPair({ bits: 2048 }, (err, keypair) => {
    const privatePem = pki.privateKeyToPem(keypair.privateKey);
    const publicPem = pki.publicKeyToPem(keypair.publicKey);

    if (!err) {
      writeFileSync(path.join(targetPath, 'private.pem'), privatePem);
      writeFileSync(path.join(targetPath, 'public.pem'), publicPem);
      spinner.stop();
      success(`generate rsa key pair success, path: ${targetPath}`);
      success('keep the key pairs carefully');
    } else {
      error('generate rsa key pair failed, please retry');
    }
  });
}
