'use strict';

const crypto = require('crypto');
const mkdirp = require('mkdirp');
const { writeFileSync, readFileSync, createReadStream } = require('fs');
const { join, relative } = require('path');
const home = require('os').homedir();
const globby = require('globby');
const gulp = require('gulp');
const zip = require('gulp-zip');

const TEMP_PATH = join(process.cwd(), '._temp');
const PACKAGE_DIR = join(process.cwd(), './_packages');
const { log, success, fatal } = require('./logger');

async function signing(config, privateKeyPath) {
  log('Signing offline-package with config below...');
  log(config);
  log('Using the private.pem: ' + privateKeyPath.replace(home, '~'));

  createManifestFile(config, TEMP_PATH);

  const artifactPaths = await globby(TEMP_PATH);
  // no tarball, only xml file
  if (artifactPaths.length === 1) {
    fatal('Artifact tarball not found!');
  }

  const CERT_JSON = {};
  for (const artifactPath of artifactPaths) {
    const artifact = relative(TEMP_PATH, artifactPath);
    const res = await doSign(artifactPath, privateKeyPath);
    CERT_JSON[artifact] = res;
  }
  writeFileSync(join(TEMP_PATH, '/CERT.json'), JSON.stringify(CERT_JSON));

  mkdirp.sync(PACKAGE_DIR);

  const zipFileName = `${config.id}_${config.version}.amr`;
  const zipFilePath = join(PACKAGE_DIR, zipFileName);
  gulp
    .src(`${TEMP_PATH}/*`)
    .pipe(zip(zipFileName))
    .pipe(gulp.dest(PACKAGE_DIR));

  success('Signed successfully at ' + zipFilePath.replace(home, '~'));
}

// ----------------------------------
// functions
// ----------------------------------
function createManifestFile(config, manifestDir) {
  try {
    const out = [];
    out.push('<?xml version="1.0" encoding="utf-8"?>');
    out.push('<package>');
    out.push('  <uid>' + config.id + '</uid>');
    out.push('  <name>' + config.id + '</name>');
    out.push('  <version>' + config.version + '</version>');
    out.push('</package>');
    const outStr = out.join('\n');
    writeFileSync(manifestDir + '/Manifest.xml', outStr);
  } catch (e) {
    fatal('Write Manifest.xml error:', e);
  }
}

async function doSign(file, privateKeyPath) {
  return new Promise(resolve => {
    const privateKey = readFileSync(privateKeyPath).toString();
    const stream = createReadStream(file);

    const sign = crypto.createSign('RSA-SHA1');

    stream.on('data', d => {
      sign.update(d);
    });

    stream.on('end', d => {
      const signture = sign.sign(privateKey, 'base64');
      resolve(signture);
    });
  });
}

module.exports = {
  signing,
};
