'use strict';

const fs = require('fs');
const OSS = require('ali-oss');
const urllib = require('urllib');
const FormData = require('form-data');
const crypto = require('crypto');
const { v4: uuidv4 } = require('uuid');

const { log } = require('./logger');
const _ = require('./common/helper');
const { appName } = require('./common/helper');
const { SOURCE_BUCKET_PATH_PREFIX } = require('./constants');

const {
  MARMOT_SERVER_URL,
  MARMOT_ACCESS_KEY_ID,
  MARMOT_ACCESS_KEY_SECRET,
  SOURCE_BUCKET_ACCESS_KEY_ID,
  SOURCE_BUCKET_ACCESS_KEY_SECRET,
  SOURCE_BUCKET_REGION,
  SOURCE_BUCKET_NAME,
} = require('./constants');

module.exports = async function(filePath) {
  if (SOURCE_BUCKET_ACCESS_KEY_ID && SOURCE_BUCKET_ACCESS_KEY_SECRET) {
    log('direct storage upload');
    return await aliyunUpload(filePath);
  }
  log('indirect storage upload');
  return await marmotUpload(filePath);
};

async function aliyunUpload(filePath) {
  const client = new OSS({
    region: SOURCE_BUCKET_REGION,
    accessKeyId: SOURCE_BUCKET_ACCESS_KEY_ID,
    accessKeySecret: SOURCE_BUCKET_ACCESS_KEY_SECRET,
    bucket: SOURCE_BUCKET_NAME,
  });
  // sample:
  // https://marmot-local-source.oss-accelerate.aliyuncs.com/source/marmot-cli/2020/8/5/e87783ea-2964-418f-a004-f903815762ae.tgz
  const date = new Date();
  const [year, month, day] = [date.getFullYear(), date.getMonth() + 1, date.getDate()];

  const storagePath = `${SOURCE_BUCKET_PATH_PREFIX}/${year}/${month}/${day}/${uuidv4()}.tgz`;
  const res = await client.put(storagePath, filePath);
  const { url } = res;
  const size = await _.getPackageFileSize(filePath);
  return {
    url,
    size,
  };
}

async function marmotUpload(filePath) {
  const form = new FormData();
  form.append('my_file', fs.createReadStream(filePath));

  const timestamp = Date.now().toString();
  const signContent = `method:POST,url:/openapi/storage_for_source?timestamp=${timestamp}`;
  const requestHeaders = Object.assign(
    {
      'User-Agent': await ua(),
    },
    form.getHeaders(),
    {
      authorization: marmotSign({
        key: MARMOT_ACCESS_KEY_ID,
        secret: MARMOT_ACCESS_KEY_SECRET,
        string: signContent,
      }),
    },
  );

  const response = await urllib.request(MARMOT_SERVER_URL + `/openapi/storage_for_source?timestamp=${timestamp}`, {
    method: 'POST',
    headers: requestHeaders,
    stream: form,
    dataType: 'json',
    timeout: [10 * 1000, 60 * 1000],
  });
  const res = response.data;

  if (res.success === false) {
    throw new Error(res.message);
  }
  const url = res.data && res.data.url;
  const size = await _.getPackageFileSize(filePath);
  return {
    url,
    size,
  };
}

function marmotSign({ key, secret, string }) {
  const signature = crypto
    .createHmac('sha256', secret)
    .update(Buffer.from(string, 'utf8'))
    .digest('base64');
  return `Marmot ${key}:${signature}`;
}

async function ua() {
  const osName = await import('os-name');
  const USER_AGENT = 'Node.js/' + process.version.slice(1) + ' (' + osName.default() + '; ' + process.arch + ')';
  if (!appName) {
    return USER_AGENT;
  }
  return appName + ' ' + USER_AGENT;
}
